@@ -5,11 +5,10 @@ use warnings;
 
 BEGIN {
 	$Reply::Plugin::TypeTiny::AUTHORITY = 'cpan:TOBYINK';
-	$Reply::Plugin::TypeTiny::VERSION   = '0.029_04';
+	$Reply::Plugin::TypeTiny::VERSION   = '0.014';
 };
 
-require Reply::Plugin;
-our @ISA = 'Reply::Plugin';
+use base 'Reply::Plugin';
 
 use Scalar::Util qw(blessed);
 use Term::ANSIColor;
@@ -18,13 +17,13 @@ sub mangle_error {
 	my $self  = shift;
 	my ($err) = @_;
 	
-	if (blessed $err and $err->isa("Error::TypeTiny::Assertion"))
+	if (blessed $err and $err->isa("Type::Exception::Assertion"))
 	{
 		my $explain = $err->explain;
 		if ($explain)
 		{
 			print color("cyan");
-			print "Error::TypeTiny::Assertion explain:\n";
+			print "Type::Exception::Assertion explain:\n";
 			$self->_explanation($explain, "");
 			local $| = 1;
 			print "\n";
@@ -84,7 +83,7 @@ L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny>.
 
 =head1 SEE ALSO
 
-L<Error::TypeTiny::Assertion>, L<Reply>.
+L<Type::Exception::Assertion>, L<Reply>.
 
 =head1 AUTHOR
 
