@@ -6,18 +6,13 @@ use warnings;
 
 BEGIN {
 	$Type::Coercion::Union::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Coercion::Union::VERSION   = '0.001';
+	$Type::Coercion::Union::VERSION   = '0.014';
 }
 
 use Scalar::Util qw< blessed >;
-use Types::TypeTiny qw< TypeTiny >;
+use Types::TypeTiny ();
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 use base "Type::Coercion";
 
@@ -25,7 +20,7 @@ sub type_coercion_map
 {
 	my $self = shift;
 	
-	TypeTiny->assert_valid(my $type = $self->type_constraint);
+	Types::TypeTiny::TypeTiny->assert_valid(my $type = $self->type_constraint);
 	$type->isa('Type::Tiny::Union')
 		or _croak "Type::Coercion::Union must be used in conjunction with Type::Tiny::Union";
 	
@@ -41,7 +36,7 @@ sub type_coercion_map
 sub add_type_coercions
 {
 	my $self = shift;
-	_croak "adding coercions to Type::Coercion::Union not currently supported";
+	_croak "Adding coercions to Type::Coercion::Union not currently supported" if @_;
 }
 
 # sub _build_moose_coercion ???
