@@ -1,25 +1,23 @@
 package Type::Exception;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
 BEGIN {
 	$Type::Exception::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Exception::VERSION   = '0.029_01';
+	$Type::Exception::VERSION   = '0.014';
 }
 
 use overload
 	q[""]    => sub { $_[0]->to_string },
-	q[bool]  => sub { 1 },
 	fallback => 1,
 ;
 
 our %CarpInternal;
 $CarpInternal{$_}++ for qw(
 	Eval::TypeTiny
-	Eval::TypeTiny::Sandbox
-	Exporter::Tiny
+	Exporter::TypeTiny
 	Test::TypeTiny
 	Type::Coercion
 	Type::Coercion::Union
@@ -53,7 +51,8 @@ sub throw
 	
 	my ($level, @caller, %ctxt) = 0;
 	while (
-		defined scalar caller($level) and $CarpInternal{scalar caller($level)}
+		(defined scalar caller($level) and $CarpInternal{scalar caller($level)})
+		or ( (caller($level))[0] =~ /^Eval::TypeTiny::/ )
 	) { $level++ };
 	if ( ((caller($level - 1))[1]||"") =~ /^parameter validation for '(.+?)'$/ )
 	{
@@ -91,7 +90,7 @@ sub to_string
 	my $m = $e->message;
 	
 	$m =~ /\n\z/s ? $m :
-	$c            ? sprintf("%s at %s line %s.\n", $m, $c->{file}||'file?', $c->{line}||'NaN') :
+	$c            ? sprintf("%s at %s line %d.\n", $m, $c->{file}, $c->{line}) :
 	sprintf("%s\n", $m);
 }
 
