@@ -1,16 +1,15 @@
 package Type::Exception::Compilation;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
 BEGIN {
 	$Type::Exception::Compilation::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Exception::Compilation::VERSION   = '0.029_01';
+	$Type::Exception::Compilation::VERSION   = '0.014';
 }
 
-require Type::Exception;
-our @ISA = 'Type::Exception';
+use base "Type::Exception";
 
 sub code        { $_[0]{code} };
 sub environment { $_[0]{environment} ||= {} };
