@@ -1,16 +1,15 @@
 package Type::Exception::WrongNumberOfParameters;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
 BEGIN {
 	$Type::Exception::WrongNumberOfParameters::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Exception::WrongNumberOfParameters::VERSION   = '0.029_01';
+	$Type::Exception::WrongNumberOfParameters::VERSION   = '0.014';
 }
 
-require Type::Exception;
-our @ISA = 'Type::Exception';
+use base "Type::Exception";
 
 sub minimum    { $_[0]{minimum} };
 sub maximum    { $_[0]{maximum} };
