@@ -1,41 +1,56 @@
 package Type::Params;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
-BEGIN {
-	if ($] < 5.008) { require Devel::TypeTiny::Perl56Compat };
-}
-
 BEGIN {
 	$Type::Params::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Params::VERSION   = '0.029_04';
+	$Type::Params::VERSION   = '0.014';
 }
 
 use B qw(perlstring);
 use Eval::TypeTiny;
 use Scalar::Util qw(refaddr);
-use Error::TypeTiny;
-use Error::TypeTiny::Assertion;
-use Error::TypeTiny::WrongNumberOfParameters;
+use Type::Exception;
+use Type::Exception::Assertion;
+use Type::Exception::WrongNumberOfParameters;
 use Type::Tiny::Union;
 use Types::Standard -types;
-use Types::TypeTiny qw(CodeLike ArrayLike to_TypeTiny);
-
-require Exporter::Tiny;
-our @ISA = 'Exporter::Tiny';
+use Types::TypeTiny qw(to_TypeTiny);
 
+use base qw< Exporter::TypeTiny >;
 our @EXPORT = qw( compile );
-our @EXPORT_OK = qw( multisig validate Invocant );
+our @EXPORT_OK = qw( validate Invocant );
 
-BEGIN {
-	my $Invocant = 'Type::Tiny::Union'->new(
-		name             => 'Invocant',
-		type_constraints => [Object, ClassName],
-	);
-	sub Invocant () { $Invocant };
-};
+use constant Invocant => "Type::Tiny::Union"->new(
+	name             => "Invocant",
+	type_constraints => [Object, ClassName],
+);
+
+#sub _exporter_expand_sub
+#{
+#	my $class = shift;
+#	my ($name, $value, $globals, $permitted) = @_;
+#	$permitted ||= $class->_exporter_permitted_regexp($globals);
+#	
+#	my %opts;
+#	if ($name eq 'compile' and keys %opts)
+#	{
+#		return compile => sub { unshift @_, \%opts; goto \&compile };
+#	}
+#	elsif ($name eq 'validate' and keys %opts)
+#	{
+#		my %compiled;
+#		return validate => sub {
+#			my $arr = shift;
+#			($compiled{ join ":", map($_->{uniq}||"\@$_->{slurpy}", @_) } ||= compile({ caller_level => 1, %opts }, @_))
+#				->(@$arr);
+#		};
+#	}
+#
+#	return $class->SUPER::_exporter_expand_sub(@_);
+#}
 
 sub _mkslurpy
 {
@@ -47,10 +62,10 @@ sub _mkslurpy
 			$i,
 		)
 		: sprintf(
-			'%s = (($#_-%d)%%2)==0 ? "Error::TypeTiny::WrongNumberOfParameters"->throw(message => sprintf("Odd number of elements in %%s", %s)) : +{ @_[%d..$#_] };',
+			'%s = (($#_-%d)%%2)==0 ? "Type::Exception::WrongNumberOfParameters"->throw(message => "Odd number of elements in %s") : +{ @_[%d..$#_] };',
 			$name,
 			$i,
-			perlstring("$tc"),
+			$tc,
 			$i,
 			$i,
 		);
@@ -91,14 +106,14 @@ sub compile
 				$constraint->is_a_type_of(Tuple)    ? _mkslurpy('$_', '@', $constraint => $arg) :
 				$constraint->is_a_type_of(HashRef)  ? _mkslurpy('$_', '%', $constraint => $arg) :
 				$constraint->is_a_type_of(ArrayRef) ? _mkslurpy('$_', '@', $constraint => $arg) :
-				Error::TypeTiny::croak("Slurpy parameter not of type HashRef or ArrayRef");
+				Type::Exception::croak("Slurpy parameter not of type HashRef or ArrayRef");
 			$varname = '$_';
 			$is_slurpy++;
 			$saw_slurpy++;
 		}
 		else
 		{
-			Error::TypeTiny::croak("Parameter following slurpy parameter") if $saw_slurpy;
+			Type::Exception::croak("Parameter following slurpy parameter") if $saw_slurpy;
 			
 			$is_optional = grep $_->{uniq} == Optional->{uniq}, $constraint->parents;
 			
@@ -110,7 +125,7 @@ sub compile
 			}
 			else
 			{
-				Error::TypeTiny::croak("Non-Optional parameter following Optional parameter") if $saw_opt;
+				Type::Exception::croak("Non-Optional parameter following Optional parameter") if $saw_opt;
 				$min_args++;
 				$max_args++;
 			}
@@ -165,11 +180,11 @@ sub compile
 		
 		push @code, sprintf 'push @R, %s;', $varname;
 	}
-	
+
 	if ($min_args == $max_args and not $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ != %d;',
+			'"Type::Exception::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ != %d;',
 			$min_args,
 			$max_args,
 			$min_args,
@@ -178,7 +193,7 @@ sub compile
 	elsif ($min_args < $max_args and not $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ < %d || @_ > %d;',
+			'"Type::Exception::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d, maximum => %d) if @_ < %d || @_ > %d;',
 			$min_args,
 			$max_args,
 			$min_args,
@@ -188,7 +203,7 @@ sub compile
 	elsif ($min_args and $saw_slurpy)
 	{
 		$code[1] = sprintf(
-			'"Error::TypeTiny::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d) if @_ < %d;',
+			'"Type::Exception::WrongNumberOfParameters"->throw(got => scalar(@_), minimum => %d) if @_ < %d;',
 			$min_args,
 			$min_args,
 		);
@@ -200,19 +215,11 @@ sub compile
 	
 	return $source if $options{want_source};
 	
-	my $closure = eval_closure(
+	return eval_closure(
 		source      => $source,
 		description => sprintf("parameter validation for '%s'", [caller(1+($options{caller_level}||0))]->[3] || '__ANON__'),
 		environment => \%env,
 	);
-	
-	return {
-		min_args   => $min_args,
-		max_args   => $saw_slurpy ? $max_args : undef,
-		closure    => $closure,
-	} if $options{want_details};
-	
-	return $closure;
 }
 
 my %compiled;
@@ -224,39 +231,6 @@ sub validate
 	goto $sub;
 }
 
-sub multisig
-{
-	my %options = (ref($_[0]) eq "HASH" && !$_[0]{slurpy}) ? %{+shift} : ();
-	my @multi = map {
-		CodeLike->check($_)  ? { closure => $_ } :
-		ArrayLike->check($_) ? compile({ want_details => 1 }, @$_) :
-		$_;
-	} @_;
-	
-	my @code = 'sub { my $r; ';
-	
-	for my $i (0 .. $#multi)
-	{
-		my $sig = $multi[$i];
-		my @cond;
-		push @cond, sprintf('@_ >= %s', $sig->{min_args}) if defined $sig->{min_args};
-		push @cond, sprintf('@_ <= %s', $sig->{max_args}) if defined $sig->{max_args};
-		push @code, sprintf('if (%s){', join('and', @cond)) if @cond;
-		push @code, sprintf('eval { $r = [ $multi[%d]{closure}->(@_) ] };', $i);
-		push @code, 'return(@$r) if $r;';
-		push @code, '}' if @cond;
-	}
-	
-	push @code, '"Error::TypeTiny"->throw(message => "Parameter validation failed");';
-	push @code, '}';
-	
-	eval_closure(
-		source      => \@code,
-		description => sprintf("parameter validation for '%s'", [caller(1+($options{caller_level}||0))]->[3] || '__ANON__'),
-		environment => { '@multi' => \@multi },
-	);
-}
-
 1;
 
 __END__
@@ -265,8 +239,6 @@ __END__
 
 =encoding utf-8
 
-=for stopwords evals
-
 =head1 NAME
 
 Type::Params - Params::Validate-like parameter validation using Type::Tiny type constraints and coercions
@@ -355,8 +327,6 @@ Dude, these functions are documented!
 
 =item Invocant
 
-=item multisig
-
 =end trustme
 
 =head1 COOKBOOK
@@ -541,70 +511,6 @@ Note that the coercion is specified as a string of Perl code. This is usually
 the fastest way to do it, but a coderef is also accepted. Either way, the
 value to be coerced is C<< $_ >>.
 
-=head2 Alternatives
-
-Type::Params can export a C<multisig> function that compiles multiple
-alternative signatures into one, and uses the first one that works:
-
-   state $check = multisig(
-      [ Int, ArrayRef ],
-      [ HashRef, Num ],
-      [ CodeRef ],
-   );
-   
-   my ($int, $arrayref) = $check->( 1, [] );
-   my ($hashref, $num)  = $check->( {}, 1.1 );
-   my ($code)           = $check->( sub { 1 } );
-   
-   $check->( sub { 1 }, 1.1 );  # throws an exception
-
-Coercions, slurpy parameters, etc still work.
-
-There's currently no indication of which of the multiple signatures
-succeeded.
-
-The present implementation involves compiling each signature independently,
-and trying them each (in their given order!) in an C<eval> block. The only
-slightly intelligent part is that it checks if C<< scalar(@_) >> fits into
-the signature properly (taking into account optional and slurpy parameters),
-and skips evals which couldn't possibly succeed.
-
-It's also possible to list coderefs as alternatives in C<multisig>:
-
-   state $check = multisig(
-      [ Int, ArrayRef ],
-      sub { ... },
-      [ HashRef, Num ],
-      [ CodeRef ],
-   );
-
-The coderef is expected to die if that alternative should be abandoned (and
-the next alternative tried), or return the list of accepted parameters. Here's
-a full example:
-
-   sub get_from {
-      state $check = multisig(
-         [ Int, ArrayRef ],
-         [ Str, HashRef ],
-         sub {
-            my ($meth, $obj);
-            die unless is_Object($obj);
-            die unless $obj->can($meth);
-            return ($meth, $obj);
-         },
-      );
-      my ($needle, $haystack) = $check->(@_);
-      
-      is_HashRef($haystack)  ? $haystack->{$needle} :
-      is_ArrayRef($haystack) ? $haystack->[$needle] :
-      is_Object($haystack)   ? $haystack->$needle   :
-      die;
-   }
-   
-   get_from(0, \@array);      # returns $array[0]
-   get_from('foo', \%hash);   # returns $hash{foo}
-   get_from('foo', $obj);     # returns $obj->foo
-
 =head1 COMPARISON WITH PARAMS::VALIDATE
 
 L<Type::Params> is not really a drop-in replacement for L<Params::Validate>;
