@@ -3,10 +3,10 @@ package Type::Parser;
 use strict;
 use warnings;
 
-sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.029_04';
+our $VERSION   = '0.014';
 
 # Token types
 # 
@@ -26,9 +26,7 @@ sub R_PAREN   () { "R_PAREN" };
 sub MYSTERY   () { "MYSTERY" };
 
 our @EXPORT_OK = qw( eval_type _std_eval parse );
-
-require Exporter::Tiny;
-our @ISA = 'Exporter::Tiny';
+use base "Exporter::TypeTiny";
 
 Evaluate: {
 	
@@ -133,14 +131,12 @@ Evaluate: {
 			my $r;
 			if ($t =~ /^(.+)::(\w+)$/)
 			{
-				require Types::TypeTiny;
 				my $library = $1; $t = $2;
 				eval "require $library;";
-				$r =
-					$library->isa('MooseX::Types::Base')  ? Types::TypeTiny::to_TypeTiny( $library->has_type($t) ? $library->can($t)->() : () ) :
-					$library->can("get_type")             ? $library->get_type($t) :
-					$reg->simple_lookup("$library\::$t", 1);
-				}
+				$r = $library->can("get_type")
+					? $library->get_type($t)
+					: $reg->simple_lookup("$library\::$t", 1);
+			}
 			else
 			{
 				$r = $reg->simple_lookup($t, 1);
@@ -206,10 +202,10 @@ Evaluate: {
 	}
 	
 	my %precedence = (
-		Type::Parser::COMMA()     , 1 ,
-		Type::Parser::UNION()     , 2 ,
-		Type::Parser::INTERSECT() , 3 ,
-		Type::Parser::NOT()       , 4 ,
+		+Type::Parser::COMMA     => 1,
+		+Type::Parser::UNION     => 2,
+		+Type::Parser::INTERSECT => 3,
+		+Type::Parser::NOT       => 4,
 	);
 	
 	sub _parse_primary
@@ -349,7 +345,6 @@ Evaluate: {
 	package # hide from CPAN
 	Type::Parser::TokenStream;
 	
-	use Scalar::Util qw(looks_like_number);
 	use Text::Balanced qw(extract_quotelike);
 	
 	sub new
@@ -466,7 +461,7 @@ Evaluate: {
 			return bless([ Type::Parser::QUOTELIKE, $quotelike ], "Type::Parser::Token"),;
 		}
 		
-		if ($self->{remaining} =~ /^([+-]?[\w:.+]+)/sm)
+		if ($self->{remaining} =~ /^([\w:]+)/sm)
 		{
 			my $spelling = $1;
 			substr($self->{remaining}, 0, length $spelling) = "";
@@ -475,10 +470,6 @@ Evaluate: {
 			{
 				return bless([ Type::Parser::CLASS, $spelling ], "Type::Parser::Token"),;
 			}
-			elsif (looks_like_number($spelling))
-			{
-				return bless([ Type::Parser::STRING, $spelling ], "Type::Parser::Token"),;
-			}
 			elsif ($self->{remaining} =~ /^\s*=>/sm) # peek ahead
 			{
 				return bless([ Type::Parser::STRING, $spelling ], "Type::Parser::Token"),;
