@@ -6,25 +6,28 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION   = '0.001';
+	$Type::Tiny::VERSION   = '0.014';
 }
 
-use Scalar::Util qw< blessed weaken refaddr isweak >;
-use Types::TypeTiny qw< StringLike CodeLike TypeTiny to_TypeTiny >;
+use Eval::TypeTiny ();
+use Scalar::Util qw( blessed weaken refaddr isweak );
+use Types::TypeTiny ();
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 sub _swap { $_[2] ? @_[1,0] : @_[0,1] }
 
+BEGIN {
+	($] > 5.010001)
+		? eval q{ sub SUPPORT_SMARTMATCH () { !!0 } }
+		: eval q{ sub SUPPORT_SMARTMATCH () { !!1 } }
+}
+
 use overload
 	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? overload::StrVal($_[0]) : $_[0]->display_name },
 	q(bool)    => sub { 1 },
 	q(&{})     => "_overload_coderef",
+	q(+)       => sub { $_[2] ? $_[1]->plus_coercions($_[0]) : $_[0]->plus_fallback_coercions($_[1]) },
 	q(|)       => sub { my @tc = _swap @_; require Type::Tiny::Union; "Type::Tiny::Union"->new(type_constraints => \@tc) },
 	q(&)       => sub { my @tc = _swap @_; require Type::Tiny::Intersection; "Type::Tiny::Intersection"->new(type_constraints => \@tc) },
 	q(~)       => sub { shift->complementary_type },
@@ -33,25 +36,41 @@ use overload
 	q(>)       => sub { my $m = $_[0]->can('is_subtype_of'); $m->(reverse _swap @_) },
 	q(<=)      => sub { my $m = $_[0]->can('is_a_type_of');  $m->(_swap @_) },
 	q(>=)      => sub { my $m = $_[0]->can('is_a_type_of');  $m->(reverse _swap @_) },
+	q(eq)      => sub { $_[2] ? ("$_[1]" eq "$_[0]") : ("$_[0]" eq "$_[1]") },
+	q(cmp)     => sub { $_[2] ? ("$_[1]" cmp "$_[0]") : ("$_[0]" cmp "$_[1]") },
 	fallback   => 1,
 ;
 BEGIN {
 	overload->import(q(~~) => sub { $_[0]->check($_[1]) })
-		if $] >= 5.010001;
+		if Type::Tiny::SUPPORT_SMARTMATCH;
 }
 
 sub _overload_coderef
 {
 	my $self = shift;
 	$self->message unless exists $self->{message};
-	$self->{_overload_coderef} ||=
-		$self->has_parent && $self->_is_null_constraint
-			? $self->parent->_overload_coderef :
-		$self->{_default_message} && "Sub::Quote"->can("quote_sub") && $self->can_be_inlined
-			? Sub::Quote::quote_sub($self->inline_assert('$_[0]')) :
-		sub { $self->assert_valid(@_) }
+	
+#	if ($self->has_parent && $self->_is_null_constraint)
+#	{
+#		$self->{_overload_coderef} ||= $self->parent->_overload_coderef;
+#	}
+#	els
+	if ($self->{_default_message} && "Sub::Quote"->can("quote_sub") && $self->can_be_inlined)
+	{
+		$self->{_overload_coderef} = Sub::Quote::quote_sub($self->inline_assert('$_[0]'))
+			if !$self->{_overload_coderef} || !$self->{_sub_quoted}++;
+	}
+	else
+	{
+		$self->{_overload_coderef} ||= sub { $self->assert_return(@_) };
+	}
+	
+	$self->{_overload_coderef};
 }
 
+our %ALL_TYPES;
+
+my $QFS;
 my $uniq = 1;
 sub new
 {
@@ -60,30 +79,64 @@ sub new
 	
 	if (exists $params{parent})
 	{
-		$params{parent} = to_TypeTiny($params{parent});
+		$params{parent} = ref($params{parent}) =~ /^Type::Tiny\b/
+			? $params{parent}
+			: Types::TypeTiny::to_TypeTiny($params{parent});
 		
-		_croak "parent must be an instance of %s", __PACKAGE__
+		_croak "Parent must be an instance of %s", __PACKAGE__
 			unless blessed($params{parent}) && $params{parent}->isa(__PACKAGE__);
 	}
 	
 	$params{name} = "__ANON__" unless exists $params{name};
 	$params{uniq} = $uniq++;
 	
-	my $self = bless \%params, $class;
-	
-	unless ($self->is_anon)
+	if ($params{name} ne "__ANON__")
 	{
 		# First try a fast ASCII-only expression, but fall back to Unicode
-		$self->name =~ /^[A-Z][A-Za-z0-9_]+$/sm
-			or eval q( $self->name =~ /^\p{Lu}[\p{L}0-9_]+$/sm )
-			or _croak '"%s" is not a valid type name', $self->name;
+		$params{name} =~ /^[A-Z][A-Za-z0-9_]+$/sm
+			or eval q( $params{name} =~ /^\p{Lu}[\p{L}0-9_]+$/sm )
+			or _croak '"%s" is not a valid type name', $params{name};
 	}
-
-	if ($self->has_library and not $self->is_anon)
+	
+	if (exists $params{coercion} and !ref $params{coercion} and $params{coercion})
 	{
-		$Moo::HandleMoose::TYPE_MAP{overload::StrVal($self)} = sub { $self->moose_type };
-	}
+		$params{parent}->has_coercion
+			or _croak "coercion => 1 requires type to have a direct parent with a coercion";
 		
+		$params{coercion} = $params{parent}->coercion;
+	}
+	
+	if (!exists $params{inlined}
+	and exists $params{constraint}
+	and ( !exists $params{parent} or $params{parent}->can_be_inlined )
+	and $QFS ||= "Sub::Quote"->can("quoted_from_sub"))
+	{
+		my (undef, $perlstring, $captures) = @{ $QFS->($params{constraint}) || [] };
+		
+		$params{inlined} = sub {
+			my ($self, $var) = @_;
+			my $code = Sub::Quote::inlinify(
+				$var eq q($_) ? $perlstring : "local \$_ = $var; $perlstring",
+				$var,
+			);
+			$code = sprintf('%s and %s', $self->parent->inline_check($var), $code) if $self->has_parent;
+			return $code;
+		} if $perlstring && !$captures;
+	}
+	
+	my $self = bless \%params, $class;
+	
+	unless ($params{tmp})
+	{
+		$Moo::HandleMoose::TYPE_MAP{overload::StrVal($self)} = sub { $self }
+			if $self->has_library && !$self->is_anon;
+		
+		$ALL_TYPES{ $self->{uniq} } = $self;
+		weaken($ALL_TYPES{ $self->{uniq} });
+	}
+	
+	$self->{type_constraints} ||= undef;
+	
 	return $self;
 }
 
@@ -95,34 +148,60 @@ sub _clone
 	$self->create_child_type(%opts);
 }
 
+sub _dd
+{
+	require B;
+	
+	my $value = @_ ? $_[0] : $_;
+	
+	!defined $value      ? 'Undef' :
+	!ref $value          ? sprintf('Value %s', B::perlstring($value)) :
+	do {
+		require Data::Dumper;
+		local $Data::Dumper::Indent   = 0;
+		local $Data::Dumper::Useqq    = 1;
+		local $Data::Dumper::Terse    = 1;
+		local $Data::Dumper::Sortkeys = 1;
+		local $Data::Dumper::Maxdepth = 2;
+		Data::Dumper::Dumper($value)
+	}
+}
+
 sub name                     { $_[0]{name} }
 sub display_name             { $_[0]{display_name}   ||= $_[0]->_build_display_name }
 sub parent                   { $_[0]{parent} }
 sub constraint               { $_[0]{constraint}     ||= $_[0]->_build_constraint }
-sub compiled_check           { $_[0]{compiled_check} ||= $_[0]->_build_compiled_check }
+sub compiled_check           { $_[0]{compiled_type_constraint} ||= $_[0]->_build_compiled_check }
 sub coercion                 { $_[0]{coercion}       ||= $_[0]->_build_coercion }
-sub message                  { $_[0]{message}        ||= $_[0]->_build_message }
+sub message                  { $_[0]{message} }
 sub library                  { $_[0]{library} }
 sub inlined                  { $_[0]{inlined} }
 sub constraint_generator     { $_[0]{constraint_generator} }
 sub inline_generator         { $_[0]{inline_generator} }
 sub name_generator           { $_[0]{name_generator} ||= $_[0]->_build_name_generator }
+sub coercion_generator       { $_[0]{coercion_generator} }
 sub parameters               { $_[0]{parameters} }
 sub moose_type               { $_[0]{moose_type}     ||= $_[0]->_build_moose_type }
 sub mouse_type               { $_[0]{mouse_type}     ||= $_[0]->_build_mouse_type }
+sub deep_explanation         { $_[0]{deep_explanation} }
 
 sub has_parent               { exists $_[0]{parent} }
 sub has_library              { exists $_[0]{library} }
-sub has_coercion             { exists $_[0]{coercion} }
+sub has_coercion             { exists $_[0]{coercion} and !!@{ $_[0]{coercion}->type_coercion_map } }
 sub has_inlined              { exists $_[0]{inlined} }
 sub has_constraint_generator { exists $_[0]{constraint_generator} }
 sub has_inline_generator     { exists $_[0]{inline_generator} }
+sub has_coercion_generator   { exists $_[0]{coercion_generator} }
 sub has_parameters           { exists $_[0]{parameters} }
+sub has_message              { exists $_[0]{message} }
+sub has_deep_explanation     { exists $_[0]{deep_explanation} }
+
+sub _default_message         { $_[0]{_default_message} ||= $_[0]->_build_default_message }
 
 sub _assert_coercion
 {
 	my $self = shift;
-	_croak "no coercion for this type constraint"
+	_croak "No coercion for this type constraint"
 		unless $self->has_coercion && @{$self->coercion->type_coercion_map};
 	return $self->coercion;
 }
@@ -148,16 +227,17 @@ sub _build_coercion
 {
 	require Type::Coercion;
 	my $self = shift;
-	return "Type::Coercion"->new(type_constraint => $self);
+	my %opts = (type_constraint => $self);
+	$opts{display_name} = "to_$self" unless $self->is_anon;
+	return "Type::Coercion"->new(%opts);
 }
 
-sub _build_message
+sub _build_default_message
 {
 	my $self = shift;
-	$self->{_default_message}++;
-	return sub { sprintf 'value "%s" did not pass type constraint', $_[0] } if $self->is_anon;
+	return sub { sprintf '%s did not pass type constraint', _dd($_[0]) } if "$self" eq "__ANON__";
 	my $name = "$self";
-	return sub { sprintf 'value "%s" did not pass type constraint "%s"', $_[0], $name };
+	return sub { sprintf '%s did not pass type constraint "%s"', _dd($_[0]), $name };
 }
 
 sub _build_name_generator
@@ -178,20 +258,31 @@ sub _build_compiled_check
 		return $self->parent->compiled_check;
 	}
 	
-	if ($self->can_be_inlined)
+	if ($INC{'Mouse/Util.pm'} and Mouse::Util::MOUSE_XS())
 	{
-		local $@;
-		my $sub = eval sprintf('sub ($) { %s }', $self->inline_check('$_[0]'));
-		die "Failed to compile check for $self: $@\n\nCODE: ".$self->inline_check('$_[0]') if $@;
-		return $sub;
+		require Mouse::Util::TypeConstraints;
+		
+		if ($self->{_is_core})
+		{
+			my $xs = "Mouse::Util::TypeConstraints"->can($self->name);
+			return $xs if $xs;
+		}
+		elsif ($self->is_parameterized and $self->has_parent
+		and $self->parent->{_is_core} and $self->parent->name =~ /^(ArrayRef|HashRef|Maybe)$/)
+		{
+			my $xs = "Mouse::Util::TypeConstraints"->can("_parameterize_".$self->parent->name."_for");
+			return $xs->($self->parameters->[0]) if $xs;
+		}
 	}
 	
-	my @constraints =
-		reverse
-		map  { $_->constraint }
-		grep { not $_->_is_null_constraint }
-		($self, $self->parents);
+	return Eval::TypeTiny::eval_closure(
+		source      => sprintf('sub ($) { %s }', $self->inline_check('$_[0]')),
+		description => sprintf("compiled check '%s'", $self),
+	) if $self->can_be_inlined;
 	
+	my @constraints;
+	push @constraints, $self->parent->compiled_check if $self->has_parent;
+	push @constraints, $self->constraint if !$self->_is_null_constraint;
 	return $null_constraint unless @constraints;
 	
 	return sub ($)
@@ -207,7 +298,7 @@ sub _build_compiled_check
 
 sub equals
 {
-	my ($self, $other) = map to_TypeTiny($_), @_;
+	my ($self, $other) = map Types::TypeTiny::to_TypeTiny($_), @_;
 	return unless blessed($self)  && $self->isa("Type::Tiny");
 	return unless blessed($other) && $other->isa("Type::Tiny");
 	
@@ -229,7 +320,7 @@ sub equals
 
 sub is_subtype_of
 {
-	my ($self, $other) = map to_TypeTiny($_), @_;
+	my ($self, $other) = map Types::TypeTiny::to_TypeTiny($_), @_;
 	return unless blessed($self)  && $self->isa("Type::Tiny");
 	return unless blessed($other) && $other->isa("Type::Tiny");
 
@@ -244,7 +335,7 @@ sub is_subtype_of
 
 sub is_supertype_of
 {
-	my ($self, $other) = map to_TypeTiny($_), @_;
+	my ($self, $other) = map Types::TypeTiny::to_TypeTiny($_), @_;
 	return unless blessed($self)  && $self->isa("Type::Tiny");
 	return unless blessed($other) && $other->isa("Type::Tiny");
 	
@@ -253,7 +344,7 @@ sub is_supertype_of
 
 sub is_a_type_of
 {
-	my ($self, $other) = map to_TypeTiny($_), @_;
+	my ($self, $other) = map Types::TypeTiny::to_TypeTiny($_), @_;
 	return unless blessed($self)  && $self->isa("Type::Tiny");
 	return unless blessed($other) && $other->isa("Type::Tiny");
 	
@@ -263,13 +354,9 @@ sub is_a_type_of
 sub qualified_name
 {
 	my $self = shift;
-	
-	if ($self->has_library and not $self->is_anon)
-	{
-		return sprintf("%s::%s", $self->library, $self->name);
-	}
-	
-	return $self->name;
+	(exists $self->{library} and $self->name ne "__ANON__")
+		? "$self->{library}::$self->{name}"
+		: $self->{name};
 }
 
 sub is_anon
@@ -288,20 +375,42 @@ sub parents
 sub check
 {
 	my $self = shift;
-	$self->compiled_check->(@_);
+	($self->{compiled_type_constraint} ||= $self->_build_compiled_check)->(@_);
+}
+
+sub _strict_check
+{
+	my $self = shift;
+	local $_ = $_[0];
+
+	my @constraints =
+		reverse
+		map  { $_->constraint }
+		grep { not $_->_is_null_constraint }
+		($self, $self->parents);
+	
+	for my $c (@constraints)
+	{
+		return unless $c->(@_);
+	}
+	
+	return !!1;
 }
 
 sub get_message
 {
 	my $self = shift;
-	$self->message->(@_);
+	local $_ = $_[0];
+	$self->has_message
+		? $self->message->(@_)
+		: $self->_default_message->(@_);
 }
 
 sub validate
 {
 	my $self = shift;
 	
-	return undef if $self->compiled_check->(@_);
+	return undef if ($self->{compiled_type_constraint} ||= $self->_build_compiled_check)->(@_);
 	
 	local $_ = $_[0];
 	return $self->get_message(@_);
@@ -311,10 +420,20 @@ sub assert_valid
 {
 	my $self = shift;
 	
-	return !!1 if $self->compiled_check->(@_);
+	return !!1 if ($self->{compiled_type_constraint} ||= $self->_build_compiled_check)->(@_);
 	
 	local $_ = $_[0];
-	_croak $self->get_message(@_);
+	$self->_failed_check("$self", $_);
+}
+
+sub assert_return
+{
+	my $self = shift;
+	
+	return $_[0] if ($self->{compiled_type_constraint} ||= $self->_build_compiled_check)->(@_);
+	
+	local $_ = $_[0];
+	$self->_failed_check("$self", $_);
 }
 
 sub can_be_inlined
@@ -330,7 +449,7 @@ sub can_be_inlined
 sub inline_check
 {
 	my $self = shift;
-	_croak "cannot inline type constraint check for %s", $self
+	_croak 'Cannot inline type constraint check for "%s"', $self
 		unless $self->can_be_inlined;
 	return $self->parent->inline_check(@_)
 		if $self->has_parent && $self->_is_null_constraint;
@@ -342,20 +461,46 @@ sub inline_check
 
 sub inline_assert
 {
+	require B;
 	my $self = shift;
 	my $varname = $_[0];
 	my $code = sprintf(
-		q[die qq(value "%s" did not pass type constraint "%s") unless %s;],
-		$varname,
-		"$self",
+		q[do { no warnings "void"; %s ? %s : Type::Tiny::_failed_check(%d, %s, %s) };],
 		$self->inline_check(@_),
+		$varname,
+		$self->{uniq},
+		B::perlstring("$self"),
+		$varname,
 	);
 	return $code;
 }
 
-sub _inline_check
+sub _failed_check
 {
-	shift->inline_check(@_);
+	require Type::Exception::Assertion;
+	
+	my ($self, $name, $value, %attrs) = @_;
+	$self = $ALL_TYPES{$self} unless ref $self;
+	
+	my $exception_class = delete($attrs{exception_class}) || "Type::Exception::Assertion";
+	
+	if ($self)
+	{
+		$exception_class->throw(
+			message => $self->get_message($value),
+			type    => $self,
+			value   => $value,
+			%attrs,
+		);
+	}
+	else
+	{
+		$exception_class->throw(
+			message => sprintf('%s did not pass type constraint "%s"', _dd($value), $name),
+			value   => $value,
+			%attrs,
+		);
+	}
 }
 
 sub coerce
@@ -380,26 +525,65 @@ sub is_parameterized
 	shift->has_parameters;
 }
 
+my %param_cache;
 sub parameterize
 {
 	my $self = shift;
 	return $self unless @_;
 	$self->is_parameterizable
-		or _croak "type '%s' does not accept parameters", "$self";
+		or _croak "Type '%s' does not accept parameters", "$self";
 	
-	@_ = map to_TypeTiny($_), @_;
+	@_ = map Types::TypeTiny::to_TypeTiny($_), @_;
 	
+	# Generate a key for caching parameterized type constraints,
+	# but only if all the parameters are strings or type constraints.
+	my $key;
+	if ( not grep(ref($_) && !Types::TypeTiny::TypeTiny->check($_), @_) )
+	{
+		require B;
+		$key = join ":", map(Types::TypeTiny::TypeTiny->check($_) ? $_->{uniq} : B::perlstring($_), $self, @_);
+	}
+	
+	return $param_cache{$key} if defined $key && defined $param_cache{$key};
+	
+	local $Type::Tiny::parameterize_type = $self;
 	local $_ = $_[0];
-	my %options = (
-		constraint   => $self->constraint_generator->(@_),
-		display_name => $self->name_generator->($self, @_),
-		parameters   => [@_],
-	);
-	$options{inlined} = $self->inline_generator->(@_)
-		if $self->has_inline_generator;
-	delete $options{inlined} unless defined $options{inlined};
+	my $P;
 	
-	return $self->create_child_type(%options);
+	my $constraint = $self->constraint_generator->(@_);
+	
+	if (Types::TypeTiny::TypeTiny->check($constraint))
+	{
+		$P = $constraint;
+	}
+	else
+	{
+		my %options = (
+			constraint   => $constraint,
+			display_name => $self->name_generator->($self, @_),
+			parameters   => [@_],
+		);
+		$options{inlined} = $self->inline_generator->(@_)
+			if $self->has_inline_generator;
+		exists $options{$_} && !defined $options{$_} && delete $options{$_}
+			for keys %options;
+		
+		$P = $self->create_child_type(%options);
+		
+		my $coercion;
+		$coercion = $self->coercion_generator->($self, $P, @_)
+			if $self->has_coercion_generator;
+		$P->coercion->add_type_coercions( @{$coercion->type_coercion_map} )
+			if $coercion;
+	}
+	
+	if (defined $key)
+	{
+		$param_cache{$key} = $P;
+		weaken($param_cache{$key});
+	}
+	
+	return $P;
 }
 
 sub child_type_class
@@ -462,7 +646,7 @@ sub _build_moose_type
 		$opts{name}       = $self->qualified_name     if $self->has_library && !$self->is_anon;
 		$opts{parent}     = $self->parent->moose_type if $self->has_parent;
 		$opts{constraint} = $self->constraint         unless $self->_is_null_constraint;
-		$opts{message}    = $self->message;
+		$opts{message}    = $self->message            if $self->has_message;
 		$opts{inlined}    = $self->inlined            if $self->has_inlined;
 		
 		$r = $self->_instantiate_moose_type(%opts);
@@ -482,14 +666,14 @@ sub _build_mouse_type
 	$options{name}       = $self->qualified_name     if $self->has_library && !$self->is_anon;
 	$options{parent}     = $self->parent->mouse_type if $self->has_parent;
 	$options{constraint} = $self->constraint         unless $self->_is_null_constraint;
-	$options{message}    = $self->message;
+	$options{message}    = $self->message            if $self->has_message;
 		
 	require Mouse::Meta::TypeConstraint;
 	my $r = "Mouse::Meta::TypeConstraint"->new(%options);
 	
 	$self->{mouse_type} = $r;  # prevent recursion
 	$r->_add_type_coercions(
-		$self->coercion->_codelike_type_coercion_map('mouse_type')
+		$self->coercion->freeze->_codelike_type_coercion_map('mouse_type')
 	) if $self->has_coercion;
 	
 	return $r;
@@ -511,6 +695,22 @@ sub plus_coercions
 	return $new;
 }
 
+sub plus_fallback_coercions
+{
+	my $self = shift;
+	
+	my @more = (@_==1 && blessed($_[0]) && $_[0]->can('type_coercion_map'))
+		? @{ $_[0]->type_coercion_map }
+		: (@_==1 && ref $_[0]) ? @{$_[0]} : @_;
+	
+	my $new = $self->_clone;
+	$new->coercion->add_type_coercions(
+		@{$self->coercion->type_coercion_map},
+		@more,
+	);
+	return $new;
+}
+
 sub minus_coercions
 {
 	my $self = shift;
@@ -546,14 +746,13 @@ sub no_coercions
 	shift->_clone;
 }
 
-# Monkey patch Moose::Meta::TypeConstraint to make
-# plus_coercions/minus_coercions/no_coercions available
+# Monkey patch Moose::Meta::TypeConstraint to refer to Type::Tiny
 sub _MONKEY_MAGIC
 {
 	return if $trick_done;
 	$trick_done++;
 	
-	eval q{
+	eval q{#line 1 "Type::Tiny::_MONKEY_MAGIC"
 		package #
 		Moose::Meta::TypeConstraint;
 		my $meta = __PACKAGE__->meta;
@@ -565,24 +764,34 @@ sub _MONKEY_MAGIC
 					writer    => "_set_tt_type",
 					predicate => "has_tt_type",
 					weak_ref  => 1,
-					Class::MOP::_definition_context(),
+					Class::MOP->can("_definition_context") ? Class::MOP::_definition_context() : (),
 				),
 			),
 		);
 		$meta->make_immutable(inline_constructor => 0);
 		1;
-	} or _croak("could not perform magic Moose trick: $@");
+	} or _croak("Could not perform magic Moose trick: $@");
 }
 
 sub isa
 {
 	my $self = shift;
 	
-	if ($INC{"Moose.pm"} and blessed($self) and my $r = $self->moose_type->isa(@_))
+	if ($INC{"Moose.pm"} and ref($self) and $_[0] eq 'Moose::Meta::TypeConstraint')
+	{
+		return !!1;
+	}
+	
+	if ($INC{"Moose.pm"} and ref($self) and $_[0] =~ /^Moose/ and my $r = $self->moose_type->isa(@_))
 	{
 		return $r;
 	}
-	
+
+	if ($INC{"Mouse.pm"} and ref($self) and $_[0] eq 'Mouse::Meta::TypeConstraint')
+	{
+		return !!1;
+	}
+
 	$self->SUPER::isa(@_);
 }
 
@@ -593,7 +802,7 @@ sub can
 	my $can = $self->SUPER::can(@_);
 	return $can if $can;
 	
-	if ($INC{"Moose.pm"} and blessed($self) and my $method = $self->moose_type->can(@_))
+	if ($INC{"Moose.pm"} and ref($self) and my $method = $self->moose_type->can(@_))
 	{
 		return sub { $method->(shift->moose_type, @_) };
 	}
@@ -607,7 +816,7 @@ sub AUTOLOAD
 	my ($m) = (our $AUTOLOAD =~ /::(\w+)$/);
 	return if $m eq 'DESTROY';
 	
-	if ($INC{"Moose.pm"} and blessed($self) and my $method = $self->moose_type->can($m))
+	if ($INC{"Moose.pm"} and ref($self) and my $method = $self->moose_type->can($m))
 	{
 		return $method->($self->moose_type, @_);
 	}
@@ -615,6 +824,35 @@ sub AUTOLOAD
 	_croak q[Can't locate object method "%s" via package "%s"], $m, ref($self)||$self;
 }
 
+sub DOES
+{
+	my $self = shift;
+	
+	return !!1 if  ref($self) && $_[0] =~ m{^ Type::API::Constraint (?: ::Coercible | ::Inlinable )? $}x;
+	return !!1 if !ref($self) && $_[0] eq 'Type::API::Constraint::Constructor';
+	
+	"UNIVERSAL"->can("DOES") ? $self->SUPER::DOES(@_) : $self->isa(@_);
+}
+
+# fill out Moose-compatible API
+sub inline_environment         { +{} }
+sub _inline_check              { shift->inline_check(@_) }
+sub _compiled_type_constraint  { shift->compiled_check(@_) }
+sub meta                       { _croak("Not really a Moose::Meta::TypeConstraint. Sorry!") }
+sub compile_type_constraint    { shift->compiled_check }
+sub _actually_compile_type_constraint   { shift->_build_compiled_check }
+sub hand_optimized_type_constraint      { shift->{hand_optimized_type_constraint} }
+sub has_hand_optimized_type_constraint  { exists(shift->{hand_optimized_type_constraint}) }
+
+# some stuff for Mouse-compatible API
+sub __is_parameterized         { shift->is_parameterized(@_) }
+sub _add_type_coercions        { shift->coercion->add_type_coercions(@_) };
+sub _as_string                 { shift->qualified_name(@_) }
+sub _compiled_type_coercion    { shift->coercion->compiled_coercion(@_) };
+sub _identity                  { refaddr(shift) };
+sub _unite                     { require Type::Tiny::Union; "Type::Tiny::Union"->new(type_constraints => \@_) };
+sub type_parameter             { my @p = @{ shift->parameters || [] }; @p==1 ? $p[0] : @p }
+
 1;
 
 __END__
@@ -623,6 +861,8 @@ __END__
 
 =encoding utf-8
 
+=for stopwords Moo(se)-compatible MooseX MouseX MooX Moose-compat
+
 =head1 NAME
 
 Type::Tiny - tiny, yet Moo(se)-compatible type constraint
@@ -650,7 +890,7 @@ Type::Tiny - tiny, yet Moo(se)-compatible type constraint
    
    package Maisy {
       use Mouse;
-      has favourite_number => (is => "ro", isa => $NUM->mouse_type);
+      has favourite_number => (is => "ro", isa => $NUM);
    }
 
 =head1 DESCRIPTION
@@ -727,6 +967,9 @@ against the type constraint. Optional (there's a vaguely sensible default.)
 A coderef which returns a string of Perl code suitable for inlining this
 type. Optional.
 
+If C<constraint> (above) is a coderef generated via L<Sub::Quote>, then
+Type::Tiny I<may> be able to automatically generate C<inlined> for you.
+
 =item C<< library >>
 
 The package name of the type library this type is associated with.
@@ -740,6 +983,10 @@ A L<Type::Coercion> object associated with this type.
 Generally speaking this attribute should not be passed to the constructor;
 you should rely on the default lazily-built coercion object.
 
+You may pass C<< coercion => 1 >> to the constructor to inherit coercions
+from the constraint's parent. (This requires the parent constraint to have
+a coercion.)
+
 =item C<< complementary_type >>
 
 A complementary type for this type. For example, the complementary type
@@ -789,16 +1036,31 @@ type constraint.
 
 A coderef which generates a new inlining coderef based on parameters.
 
+=item C<< coercion_generator >>
+
+A coderef which generates a new L<Type::Coercion> object based on parameters.
+
+=item C<< deep_explanation >>
+
+This API is not finalized. Coderef used by L<Type::Exception::Assertion> to
+peek inside parameterized types and figure out why a value doesn't pass the
+constraint.
+
 =back
 
 =head2 Methods
 
 =over
 
-=item C<has_parent>, C<has_coercion>, C<has_library>, C<has_constraint_generator>, C<has_inlined>, C<has_inline_generator>, C<has_parameters>
+=item C<has_parent>, C<has_library>, C<has_inlined>, C<has_constraint_generator>, C<has_inline_generator>, C<has_coercion_generator>, C<has_parameters>, C<has_message>, C<has_deep_explanation>
 
 Predicate methods.
 
+=item C<has_coercion>
+
+Predicate method with a little extra DWIM. Returns false if the coercion is
+a no-op.
+
 =item C<< is_anon >>
 
 Returns true iff the type constraint does not have a C<name>.
@@ -815,7 +1077,18 @@ C<< "Library::Type" >> sort of name. Otherwise, returns the same as C<name>.
 
 =item C<< parents >>
 
-Returns a list of all this type constraint's all ancestor constraints.
+Returns a list of all this type constraint's ancestor constraints. For
+example, if called on the C<Str> type constraint would return the list
+C<< (Value, Defined, Item, Any) >>.
+
+B<< Due to a historical misunderstanding, this differs from the Moose
+implementation of the C<parents> method. In Moose, C<parents> only returns the
+immediate parent type constraints, and because type constraints only have
+one immediate parent, this is effectively an alias for C<parent>. The
+extension module L<MooseX::Meta::TypeConstraint::Intersection> is the only
+place where multiple type constraints are returned; and they are returned
+as an arrayref in violation of the base class' documentation. I'm keeping
+my behaviour as it seems more useful. >>
 
 =item C<< equals($other) >>, C<< is_subtype_of($other) >>, C<< is_supertype_of($other) >>, C<< is_a_type_of($other) >>
 
@@ -839,6 +1112,15 @@ constraint.
 Yes, that's three very similar methods. Blame L<Moose::Meta::TypeConstraint>
 whose API I'm attempting to emulate. :-)
 
+=item C<< assert_return($value) >>
+
+Like C<< assert_valid($value) >> but returns the value if it passes the type
+constraint.
+
+This seems a more useful behaviour than C<< assert_valid($value) >>. I would
+have just changed C<< assert_valid($value) >> to do this, except that there
+are edge cases where it could break Moose compatibility.
+
 =item C<< get_message($value) >>
 
 Returns the error message for the value; even if the value passes the type
@@ -898,6 +1180,10 @@ Shorthand for creating a new child type constraint with the same coercions
 as this one, but then adding some extra coercions (at a higher priority than
 the existing ones).
 
+=item C<< plus_fallback_coercions($type1, $code1, ...) >>
+
+Like C<plus_coercions>, but added at a lower priority.
+
 =item C<< minus_coercions($type1, ...) >>
 
 Shorthand for creating a new child type constraint with fewer type coercions.
@@ -911,6 +1197,33 @@ Shorthand for creating a new child type constraint with no coercions at all.
 If Moose is loaded, then the combination of these methods is used to mock
 a Moose::Meta::TypeConstraint.
 
+If Mouse is loaded, then C<isa> mocks Mouse::Meta::TypeConstraint.
+
+=item C<< DOES($role) >>
+
+Overridden to advertise support for various roles.
+
+See also L<Type::API::Constraint>, etc.
+
+=back
+
+The following methods exist for Moose/Mouse compatibility, but do not do
+anything useful.
+
+=over
+
+=item C<< compile_type_constraint >>
+
+=item C<< hand_optimized_type_constraint >>
+
+=item C<< has_hand_optimized_type_constraint >>
+
+=item C<< inline_environment >>
+
+=item C<< meta >>
+
+=item C<< type_parameter >>
+
 =back
 
 =head2 Overloading
@@ -927,7 +1240,7 @@ Boolification is overloaded to always return true.
 
 =item *
 
-Coderefification is overloaded to call C<assert_value>.
+Coderefification is overloaded to call C<assert_return>.
 
 =item *
 
@@ -956,6 +1269,22 @@ See L<Type::Tiny::Union>.
 The C<< & >> operator is overloaded to build the intersection of two type
 constraints. See L<Type::Tiny::Intersection>.
 
+=item *
+
+The C<< + >> operator is overloaded to call C<plus_coercions> or
+C<plus_fallback_coercions> as appropriate.
+
+=back
+
+=head2 Constants
+
+=over
+
+=item C<< Type::Tiny::SUPPORT_SMARTMATCH >>
+
+Indicates whether the smart match overload is supported on your
+version of Perl.
+
 =back
 
 =head1 BUGS
@@ -965,7 +1294,7 @@ L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny>.
 
 =head1 SEE ALSO
 
-L<Type::Tiny::Manual>.
+L<Type::Tiny::Manual>, L<Type::API>.
 
 L<Type::Library>, L<Type::Utils>, L<Types::Standard>, L<Type::Coercion>.
 
@@ -975,6 +1304,8 @@ L<Type::Tiny::Enum>, L<Type::Tiny::Union>, L<Type::Tiny::Intersection>.
 L<Moose::Meta::TypeConstraint>,
 L<Mouse::Meta::TypeConstraint>.
 
+L<Type::Params>.
+
 =head1 AUTHOR
 
 Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
