@@ -6,25 +6,26 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Duck::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Duck::VERSION   = '0.001';
+	$Type::Tiny::Duck::VERSION   = '0.014';
 }
 
 use Scalar::Util qw< blessed >;
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 use base "Type::Tiny";
 
 sub new {
 	my $proto = shift;
+	
 	my %opts = @_;
-	_croak "need to supply list of methods" unless exists $opts{methods};
+	_croak "Duck type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
+	_croak "Duck type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
+	_croak "Duck type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
+	_croak "Need to supply list of methods" unless exists $opts{methods};
+	
 	$opts{methods} = [$opts{methods}] unless ref $opts{methods};
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -51,7 +52,7 @@ sub _build_inlined
 	};
 }
 
-sub _build_message
+sub _build_default_message
 {
 	my $self = shift;
 	return sub { sprintf 'value "%s" did not pass type constraint', $_[0] } if $self->is_anon;
@@ -71,6 +72,17 @@ sub _instantiate_moose_type
 	return "Moose::Meta::TypeConstraint::DuckType"->new(%opts, methods => $self->methods);
 }
 
+sub has_parent
+{
+	!!1;
+}
+
+sub parent
+{
+	require Types::Standard;
+	Types::Standard::Object();
+}
+
 1;
 
 __END__
@@ -108,6 +120,11 @@ constructor. Instead rely on the default.
 Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
 the constructor. Instead rely on the default.
 
+=item C<parent>
+
+Parent is always Types::Standard::Object, and cannot be passed to the
+constructor.
+
 =back
 
 =head1 BUGS
