@@ -6,15 +6,10 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Enum::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Enum::VERSION   = '0.001';
+	$Type::Tiny::Enum::VERSION   = '0.014';
 }
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 use overload q[@{}] => 'values';
 
@@ -23,12 +18,18 @@ use base "Type::Tiny";
 sub new
 {
 	my $proto = shift;
+	
 	my %opts = @_;
-	_croak "need to supply list of values" unless exists $opts{values};
+	_croak "Enum type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
+	_croak "Enum type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
+	_croak "Enum type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
+	_croak "Need to supply list of values" unless exists $opts{values};
+	
 	my %tmp =
 		map { $_ => 1 }
 		@{ ref $opts{values} eq "ARRAY" ? $opts{values} : [$opts{values}] };
 	$opts{values} = [sort keys %tmp];
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -73,6 +74,17 @@ sub _instantiate_moose_type
 	return "Moose::Meta::TypeConstraint::Enum"->new(%opts, values => $self->values);
 }
 
+sub has_parent
+{
+	!!1;
+}
+
+sub parent
+{
+	require Types::Standard;
+	Types::Standard::Str();
+}
+
 1;
 
 __END__
@@ -111,6 +123,11 @@ constructor. Instead rely on the default.
 Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
 the constructor. Instead rely on the default.
 
+=item C<parent>
+
+Parent is always Types::Standard::Str, and cannot be passed to the
+constructor.
+
 =back
 
 =head2 Overloading
