@@ -6,32 +6,33 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Intersection::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Intersection::VERSION   = '0.001';
+	$Type::Tiny::Intersection::VERSION   = '0.014';
 }
 
 use Scalar::Util qw< blessed >;
-use Types::TypeTiny qw< to_TypeTiny >;
+use Types::TypeTiny ();
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
-use overload q[@{}] => 'type_constraints';
+use overload q[@{}] => sub { $_[0]{type_constraints} ||= [] };
 
 use base "Type::Tiny";
 
 sub new {
 	my $proto = shift;
+	
 	my %opts = @_;
-	_croak "need to supply list of type constraints" unless exists $opts{type_constraints};
+	_croak "Intersection type constraints cannot have a parent constraint" if exists $opts{parent};
+	_croak "Intersection type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
+	_croak "Intersection type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
+	_croak "Need to supply list of type constraints" unless exists $opts{type_constraints};
+	
 	$opts{type_constraints} = [
 		map { $_->isa(__PACKAGE__) ? @$_ : $_ }
-		map to_TypeTiny($_),
+		map Types::TypeTiny::to_TypeTiny($_),
 		@{ ref $opts{type_constraints} eq "ARRAY" ? $opts{type_constraints} : [$opts{type_constraints}] }
 	];
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -67,6 +68,16 @@ sub inline_check
 	sprintf '(%s)', join " and ", map $_->inline_check($_[0]), @$self;
 }
 
+sub has_parent
+{
+	!!@{ $_[0]{type_constraints} };
+}
+
+sub parent
+{
+	$_[0]{type_constraints}[0];
+}
+
 1;
 
 __END__
@@ -108,6 +119,13 @@ constructor. Instead rely on the default.
 Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
 the constructor. Instead rely on the default.
 
+=item C<parent>
+
+Parent is automatically calculated, and cannot be passed to the constructor.
+
+(Technically any of the types in the intersection could be treated as a
+parent type; we choose the first arbitrarily.)
+
 =back
 
 =head2 Overloading
