@@ -6,24 +6,26 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Role::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Role::VERSION   = '0.001';
+	$Type::Tiny::Role::VERSION   = '0.014';
 }
 
-use Scalar::Util qw< blessed >;
+use Scalar::Util qw< blessed weaken >;
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
 use base "Type::Tiny";
 
+my %cache;
+
 sub new {
 	my $proto = shift;
+	
 	my %opts = @_;
-	_croak "need to supply role name" unless exists $opts{role};
+	_croak "Role type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
+	_croak "Role type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
+	_croak "Role type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
+	_croak "Need to supply role name" unless exists $opts{role};
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -49,7 +51,7 @@ sub _build_inlined
 	};
 }
 
-sub _build_message
+sub _build_default_message
 {
 	my $self = shift;
 	my $c = $self->role;
@@ -58,6 +60,17 @@ sub _build_message
 	return sub { sprintf 'value "%s" did not pass type constraint "%s" (not DOES %s)', $_[0], $name, $c };
 }
 
+sub has_parent
+{
+	!!1;
+}
+
+sub parent
+{
+	require Types::Standard;
+	Types::Standard::Object();
+}
+
 1;
 
 __END__
@@ -99,6 +112,11 @@ constructor. Instead rely on the default.
 Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
 the constructor. Instead rely on the default.
 
+=item C<parent>
+
+Parent is always Types::Standard::Object, and cannot be passed to the
+constructor.
+
 =back
 
 =head1 BUGS
