@@ -6,32 +6,33 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Union::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Union::VERSION   = '0.001';
+	$Type::Tiny::Union::VERSION   = '0.014';
 }
 
 use Scalar::Util qw< blessed >;
-use Types::TypeTiny qw< to_TypeTiny >;
+use Types::TypeTiny ();
 
-sub _croak ($;@)
-{
-	require Carp;
-	@_ = sprintf($_[0], @_[1..$#_]) if @_ > 1;
-	goto \&Carp::croak;
-}
+sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
 
-use overload q[@{}] => 'type_constraints';
+use overload q[@{}] => sub { $_[0]{type_constraints} ||= [] };
 
 use base "Type::Tiny";
 
 sub new {
 	my $proto = shift;
+	
 	my %opts = @_;
-	_croak "need to supply list of type constraints" unless exists $opts{type_constraints};
+	_croak "Union type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
+	_croak "Union type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
+	_croak "Union type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
+	_croak "Need to supply list of type constraints" unless exists $opts{type_constraints};
+	
 	$opts{type_constraints} = [
 		map { $_->isa(__PACKAGE__) ? @$_ : $_ }
-		map to_TypeTiny($_),
+		map Types::TypeTiny::to_TypeTiny($_),
 		@{ ref $opts{type_constraints} eq "ARRAY" ? $opts{type_constraints} : [$opts{type_constraints}] }
 	];
+	
 	my $self = $proto->SUPER::new(%opts);
 	$self->coercion if grep $_->has_coercion, @$self;
 	return $self;
@@ -90,6 +91,29 @@ sub _instantiate_moose_type
 	return "Moose::Meta::TypeConstraint::Union"->new(%opts, type_constraints => \@tc);
 }
 
+sub has_parent
+{
+	defined(shift->parent);
+}
+
+sub parent
+{
+	$_[0]{parent} ||= $_[0]->_build_parent;
+}
+
+sub _build_parent
+{
+	my $self = shift;
+	my ($first, @rest) = @$self;
+	
+	for my $parent ($first, $first->parents)
+	{
+		return $parent unless grep !$_->is_a_type_of($parent), @rest;
+	}
+	
+	return;
+}
+
 1;
 
 __END__
